from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from smart_settings import Namespace

namespace = Namespace(name='converter', label=_('Converter'))
setting_graphics_backend = namespace.add_setting(
    default='converter.backends.python.Python',
    help_text=_('Graphics conversion backend to use.'),
    global_name='CONVERTER_GRAPHICS_BACKEND',
)
setting_graphics_backend_config = namespace.add_setting(
    default='{libreoffice_path: /usr/bin/libreoffice, '
    'pdftoppm_path: /usr/bin/pdftoppm, pdfinfo_path: /usr/bin/pdfinfo}',
    help_text=_(
        'Configuration options for the graphics conversion backend.'
    ), global_name='CONVERTER_GRAPHICS_BACKEND_CONFIG',
)



from __future__ import unicode_literals

from ..widgets import DocumentThumbnailWidget

from .test_models import GenericDocumentTestCase


class DocumentWidgetTestCase(GenericDocumentTestCase):
    def test_document_list_view_document_with_no_pages(self):
        document_thumbnail_widget = DocumentThumbnailWidget()
        self.document.pages.all().delete()
        result = document_thumbnail_widget.render(instance=self.document)

        self.assertTrue(self.document.get_absolute_url() in result)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('document_signatures', '0004_auto_20160325_0418'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='documentversionsignature',
            name='document_version',
        ),
        migrations.DeleteModel(
            name='DocumentVersionSignature',
        ),
    ]



from __future__ import unicode_literals

import json
import logging

from django.core import mail
from django.db import models
from django.utils.module_loading import import_string
from django.utils.translation import ugettext_lazy as _

from .utils import split_recipient_list

logger = logging.getLogger(__name__)


class LogEntry(models.Model):
    datetime = models.DateTimeField(
        auto_now_add=True, editable=False, verbose_name=_('Date time')
    )
    message = models.TextField(
        blank=True, editable=False, verbose_name=_('Message')
    )

    class Meta:
        get_latest_by = 'datetime'
        ordering = ('-datetime',)
        verbose_name = _('Log entry')
        verbose_name_plural = _('Log entries')


class UserMailer(models.Model):
    label = models.CharField(
        max_length=128, unique=True, verbose_name=_('Label')
    )
    default = models.BooleanField(
        default=True, help_text=_(
            'If default, this mailing profile will be pre-selected on the '
            'document mailing form.'
        ), verbose_name=_('Default')
    )
    enabled = models.BooleanField(default=True, verbose_name=_('Enabled'))
    backend_path = models.CharField(
        max_length=128,
        help_text=_('The dotted Python path to the backend class.'),
        verbose_name=_('Backend path')
    )
    backend_data = models.TextField(
        blank=True, verbose_name=_('Backend data')
    )

    class Meta:
        ordering = ('label',)
        verbose_name = _('User mailer')
        verbose_name_plural = _('User mailers')

    def __str__(self):
        return self.label

    def save(self, *args, **kwargs):
        if self.default:
            UserMailer.objects.select_for_update().exclude(pk=self.pk).update(
                default=False
            )

        return super(UserMailer, self).save(*args, **kwargs)

    def backend_label(self):
        return self.get_backend().label

    def get_backend(self):
        return import_string(self.backend_path)

    def get_connection(self):
        return mail.get_connection(
            backend=self.get_backend().class_path, **self.loads()
        )

    def loads(self):
        return json.loads(self.backend_data)

    def dumps(self, data):
        self.backend_data = json.dumps(data)
        self.save()

    def send(self, subject='', body='', to=None, document=None, as_attachment=False):
        recipient_list = split_recipient_list(recipients=[to])

        with self.get_connection() as connection:
            email_message = mail.EmailMultiAlternatives(
                subject=subject, body=body, to=recipient_list,
                connection=connection
            )

            if as_attachment:
                with document.open() as descriptor:
                    email_message.attach(
                        filename=document.label, content=descriptor.read(),
                        mimetype=document.file_mimetype
                    )

            try:
                email_message.send()
            except Exception as exception:
                self.error_log.create(message=exception)
            else:
                self.error_log.all().delete()

    def test(self, to):
        self.send(to=to, subject=_('Test email from Mayan EDMS'))


class UserMailerLogEntry(models.Model):
    user_mailer = models.ForeignKey(
        UserMailer, on_delete=models.CASCADE, related_name='error_log',
        verbose_name=_('User mailer')
    )
    datetime = models.DateTimeField(
        auto_now_add=True, editable=False, verbose_name=_('Date time')
    )
    message = models.TextField(
        blank=True, editable=False, verbose_name=_('Message')
    )

    class Meta:
        get_latest_by = 'datetime'
        ordering = ('-datetime',)
        verbose_name = _('User mailer log entry')
        verbose_name_plural = _('User mailer log entries')



from __future__ import absolute_import, unicode_literals

from django.conf import settings
from django.contrib import messages
from django.contrib.auth.views import (
    login, password_change, password_reset, password_reset_confirm,
    password_reset_complete, password_reset_done
)
from django.http import HttpResponseRedirect
from django.shortcuts import redirect
from django.urls import reverse
from django.utils.translation import ugettext_lazy as _

from stronghold.decorators import public

from .forms import EmailAuthenticationForm, UsernameAuthenticationForm
from .settings import setting_login_method, setting_maximum_session_length


@public
def login_view(request):
    """
    Control how the use is to be authenticated, options are 'email' and
    'username'
    """
    kwargs = {'template_name': 'authentication/login.html'}

    if setting_login_method.value == 'email':
        kwargs['authentication_form'] = EmailAuthenticationForm
    else:
        kwargs['authentication_form'] = UsernameAuthenticationForm

    if not request.user.is_authenticated:
        extra_context = {'appearance_type': 'plain'}

        result = login(request, extra_context=extra_context, **kwargs)
        if request.method == 'POST':
            form = kwargs['authentication_form'](request, data=request.POST)
            if form.is_valid():
                if form.cleaned_data['remember_me']:
                    request.session.set_expiry(
                        setting_maximum_session_length.value
                    )
                else:
                    request.session.set_expiry(0)
        return result
    else:
        return HttpResponseRedirect(reverse(settings.LOGIN_REDIRECT_URL))


def password_change_view(request):
    """
    Password change wrapper for better control
    """
    extra_context = {'title': _('Current user password change')}

    return password_change(
        request, extra_context=extra_context,
        template_name='appearance/generic_form.html',
        post_change_redirect=reverse('authentication:password_change_done'),
    )


def password_change_done(request):
    """
    View called when the new user password has been accepted
    """
    messages.success(
        request, _('Your password has been successfully changed.')
    )
    return redirect('common:current_user_details')


@public
def password_reset_complete_view(request):
    extra_context = {
        'appearance_type': 'plain'
    }

    return password_reset_complete(
        request, extra_context=extra_context,
        template_name= 'authentication/password_reset_complete.html'
    )


@public
def password_reset_confirm_view(request, uidb64=None, token=None):
    extra_context = {
        'appearance_type': 'plain'
    }

    return password_reset_confirm(
        request, extra_context=extra_context,
        template_name='authentication/password_reset_confirm.html',
        post_reset_redirect=reverse(
            'authentication:password_reset_complete_view'
        ), uidb64=uidb64, token=token
    )


@public
def password_reset_done_view(request):
    extra_context = {
        'appearance_type': 'plain'
    }

    return password_reset_done(
        request, extra_context=extra_context,
        template_name='authentication/password_reset_done.html'
    )


@public
def password_reset_view(request):
    extra_context = {
        'appearance_type': 'plain'
    }

    return password_reset(
        request, extra_context=extra_context,
        email_template_name='authentication/password_reset_email.html',
        extra_email_context={
            'project_title': settings.PROJECT_TITLE,
            'project_website': settings.PROJECT_WEBSITE,
            'project_copyright': settings.PROJECT_COPYRIGHT,
            'project_license': settings.PROJECT_LICENSE,
        }, subject_template_name='authentication/password_reset_subject.txt',
        template_name='authentication/password_reset_form.html',
        post_reset_redirect=reverse(
            'authentication:password_reset_done_view'
        )
    )



from __future__ import unicode_literals

from django.core import mail

from documents.tests.test_views import (
    GenericDocumentViewTestCase, GenericViewTestCase
)

from ..models import UserMailer
from ..permissions import (
    permission_mailing_link, permission_mailing_send_document,
    permission_user_mailer_use, permission_user_mailer_view
)

from .literals import (
    TEST_EMAIL_ADDRESS, TEST_USER_MAILER_BACKEND_PATH, TEST_USER_MAILER_LABEL
)


class MailerTestMixin(object):
    def _create_user_mailer(self):

        self.user_mailer = UserMailer.objects.create(
            default=True,
            enabled=True,
            label=TEST_USER_MAILER_LABEL,
            backend_path=TEST_USER_MAILER_BACKEND_PATH,
            backend_data='{}'
        )


class MailerViewsTestCase(MailerTestMixin, GenericDocumentViewTestCase):
    def _request_document_link_send(self):
        return self.post(
            'mailer:send_document_link', args=(self.document.pk,),
            data={
                'email': TEST_EMAIL_ADDRESS,
                'user_mailer': self.user_mailer.pk
            },
        )

    def _request_document_send(self):
        return self.post(
            'mailer:send_document', args=(self.document.pk,),
            data={
                'email': TEST_EMAIL_ADDRESS,
                'user_mailer': self.user_mailer.pk
            },
        )

    def test_mail_link_view_no_permissions(self):
        self._create_user_mailer()
        self.login_user()

        response = self._request_document_link_send()

        self.assertContains(
            response, 'Select a valid choice', status_code=200
        )

    def test_mail_link_view_with_permission(self):
        self._create_user_mailer()
        self.login_user()

        self.grant_permission(permission=permission_mailing_link)
        self.grant_permission(permission=permission_user_mailer_use)

        self._request_document_link_send()

        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].to, [TEST_EMAIL_ADDRESS])

    def test_mail_document_view_no_permissions(self):
        self._create_user_mailer()
        self.login_user()

        response = self._request_document_send()
        self.assertContains(
            response, 'Select a valid choice', status_code=200
        )

    def test_mail_document_view_with_permission(self):
        self._create_user_mailer()
        self.login_user()

        self.grant_permission(permission=permission_mailing_send_document)
        self.grant_permission(permission=permission_user_mailer_use)

        self._request_document_send()

        self.assertEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].to, [TEST_EMAIL_ADDRESS])


class UserMailerViewTestCase(MailerTestMixin, GenericViewTestCase):
    def _request_user_mailer_delete(self):
        return self.post(
            'mailer:user_mailer_delete', args=(self.user_mailer.pk,)
        )

    def test_user_mailer_list_view_no_permissions(self):
        self._create_user_mailer()
        self.login_user()

        response = self.get(
            'mailer:user_mailer_list',
        )
        self.assertNotContains(
            response, text=self.user_mailer.label, status_code=200
        )

    def test_user_mailer_list_view_with_permissions(self):
        self._create_user_mailer()
        self.login_user()

        self.grant_permission(permission=permission_user_mailer_view)

        response = self.get(
            'mailer:user_mailer_list',
        )

        self.assertContains(
            response, text=self.user_mailer.label, status_code=200
        )

    def test_user_mailer_delete_view_no_permissions(self):
        self._create_user_mailer()
        self.login_user()

        self._request_user_mailer_delete()

        self.assertQuerysetEqual(
            UserMailer.objects.all(), (repr(self.user_mailer),)
        )

    def test_user_mailer_delete_view_with_permissions(self):
        self._create_user_mailer()
        self.login_user()

        self.grant_permission(permission=permission_user_mailer_view)

        self._request_user_mailer_delete()

        self.assertNotEqual(
            [UserMailer.objects.all()], [self.user_mailer]
        )



from __future__ import unicode_literals

from django.apps import apps

from document_indexing.tasks import task_index_document


def launch_workflow(sender, instance, created, **kwargs):
    Workflow = apps.get_model(
        app_label='document_states', model_name='Workflow'
    )

    if created:
        Workflow.objects.launch_for(instance)


def handler_index_document(sender, **kwargs):
    task_index_document.apply_async(
        kwargs=dict(
            document_id=kwargs['instance'].workflow_instance.document.pk
        )
    )



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from task_manager.classes import CeleryQueue

queue_metadata = CeleryQueue(
    name='metadata', label=_('Metadata')
)
queue_metadata.add_task_type(
    name='metadata.tasks.task_remove_metadata_type',
    label=_('Remove metadata type')
)
queue_metadata.add_task_type(
    name='metadata.tasks.task_add_required_metadata_type',
    label=_('Add required metadata type')
)



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from task_manager.classes import CeleryQueue

queue_mailing = CeleryQueue(
    name='mailing', label=_('Mailing')
)
queue_mailing.add_task_type(
    name='mailer.tasks.task_send_document',
    label=_('Send document')
)



from __future__ import unicode_literals

import datetime
from errno import ENOENT
import logging
from stat import S_IFDIR, S_IFREG
from time import time

from fuse import FUSE, FuseOSError, Operations

from django.core import management
from django.core.cache import caches
from django.core.exceptions import MultipleObjectsReturned
from django.core.management.base import CommandError
from django.db.models import Count

from document_indexing.models import Index, IndexInstanceNode
from documents.models import Document

from ...literals import (
    MAX_FILE_DESCRIPTOR, MIN_FILE_DESCRIPTOR, FILE_MODE, DIRECTORY_MODE
)
from ...settings import (
    setting_document_lookup_cache_timeout, setting_node_lookup_cache_timeout
)

logger = logging.getLogger(__name__)


class IndexFS(Operations):
    def _get_next_file_descriptor(self):
        while(True):
            self.file_descriptor_count += 1
            if self.file_descriptor_count > MAX_FILE_DESCRIPTOR:
                self.file_descriptor_count = MIN_FILE_DESCRIPTOR

            try:
                if not self.file_descriptors[self.file_descriptor_count]:
                    return self.file_descriptor_count
            except KeyError:
                return self.file_descriptor_count

    def _path_to_node(self, path, access_only=False, directory_only=True):
        logger.debug('path: %s', path)
        logger.debug('directory_only: %s', directory_only)

        parts = path.split('/')

        logger.debug('parts: %s', parts)

        node = self.index.instance_root

        if len(parts) > 1 and parts[1] != '':
            obj = self.cache.get(path)

            if obj:
                node_pk = obj.get('node_pk')
                if node_pk:
                    if access_only:
                        return True
                    else:
                        return IndexInstanceNode.objects.get(pk=node_pk)

                document_pk = obj.get('document_pk')
                if document_pk:
                    if access_only:
                        return True
                    else:
                        return Document.objects.get(pk=document_pk)

            for count, part in enumerate(parts[1:]):
                try:
                    node = node.children.get(value=part)
                except IndexInstanceNode.DoesNotExist:
                    logger.debug('%s does not exists', part)

                    if directory_only:
                        return None
                    else:
                        try:
                            if node.index_template_node.link_documents:
                                result = node.documents.get(label=part)
                                logger.debug(
                                    'path %s is a valid file path', path
                                )
                                self.cache.set(
                                    path, {'document_pk': result.pk},
                                    setting_document_lookup_cache_timeout.value
                                )

                                return result
                            else:
                                return None
                        except Document.DoesNotExist:
                            logger.debug(
                                'path %s is a file, but is not found', path
                            )
                            return None
                        except MultipleObjectsReturned:
                            return None
                except MultipleObjectsReturned:
                    return None

            self.cache.set(
                path, {'node_pk': node.pk},
                setting_node_lookup_cache_timeout.value
            )

        logger.debug('node: %s', node)
        logger.debug('node is root: %s', node.is_root_node())

        return node

    def __init__(self, index_slug):
        self.file_descriptor_count = MIN_FILE_DESCRIPTOR
        self.file_descriptors = {}
        self.cache = caches['default']

        try:
            self.index = Index.objects.get(slug=index_slug)
        except Index.DoesNotExist:
            print 'Unknown index slug: {}.'.format(index_slug)
            exit(1)

    def access(self, path, fh=None):
        result = self._path_to_node(
            path=path, access_only=True, directory_only=False
        )

        if not result:
            raise FuseOSError(ENOENT)

    def getattr(self, path, fh=None):
        logger.debug('path: %s, fh: %s', path, fh)

        now = time()
        result = self._path_to_node(path=path, directory_only=False)

        if not result:
            raise FuseOSError(ENOENT)

        if isinstance(result, IndexInstanceNode):
            return {
                'st_mode': (S_IFDIR | DIRECTORY_MODE), 'st_ctime': now,
                'st_mtime': now, 'st_atime': now, 'st_nlink': 2
            }
        else:
            return {
                'st_mode': (S_IFREG | FILE_MODE),
                'st_ctime': (
                    result.date_added.replace(tzinfo=None) - result.date_added.utcoffset() - datetime.datetime(1970, 1, 1)
                ).total_seconds(),
                'st_mtime': (
                    result.latest_version.timestamp.replace(tzinfo=None) - result.latest_version.timestamp.utcoffset() - datetime.datetime(1970, 1, 1)
                ).total_seconds(),
                'st_atime': now,
                'st_size': result.size
            }

    def open(self, path, flags):
        result = self._path_to_node(path=path, directory_only=False)

        if isinstance(result, Document):
            next_file_descriptor = self._get_next_file_descriptor()
            self.file_descriptors[next_file_descriptor] = result.open()
            return next_file_descriptor
        else:
            raise FuseOSError(ENOENT)

    def release(self, path, fh):
        self.file_descriptors[fh] = None
        del(self.file_descriptors[fh])

    def read(self, path, size, offset, fh):
        self.file_descriptors[fh].seek(offset)
        return self.file_descriptors[fh].read(size)

    def readdir(self, path, fh):
        logger.debug('path: %s', path)

        node = self._path_to_node(path=path, directory_only=True)

        if not node:
            raise FuseOSError(ENOENT)

        yield '.'
        yield '..'

        # Nodes
        queryset = node.get_children().values('value').exclude(
            value__contains='/'
        )

        for duplicate in queryset.order_by().annotate(count_id=Count('id')).filter(count_id__gt=1):
            queryset = queryset.exclude(label=duplicate['label'])

        for child_node in queryset.values_list('value', flat=True):
            yield child_node

        # Documents
        if node.index_template_node.link_documents:
            queryset = node.documents.values('label').exclude(
                label__contains='/'
            )

            for duplicate in queryset.order_by().annotate(count_id=Count('id')).filter(count_id__gt=1):
                queryset = queryset.exclude(label=duplicate['label'])

            for document_label in queryset.values_list('label', flat=True):
                yield document_label


class Command(management.BaseCommand):
    help = 'Mount an index as a FUSE filesystem.'

    def add_arguments(self, parser):
        parser.add_argument('slug', nargs='?', help='Index slug')
        parser.add_argument('mount_point', nargs='?', help='Mount point')
        parser.add_argument(
            '--allow-other', action='store_true', dest='allow_other',
            default=False,
            help='All users (including root) can access the index files.'
        )
        parser.add_argument(
            '--allow-root', action='store_true', dest='allow_root',
            default=False,
            help='Mount access is limited to the user mounting the index and '
            'root. This option and --allow-other are mutually exclusive.'
        )

    def handle(self, *args, **options):
        if not options.get('slug') or not options.get('mount_point'):
            self.stderr.write(self.style.ERROR('Incorrect number of arguments'))
            exit(1)

        try:
            FUSE(
                operations=IndexFS(index_slug=options['slug']),
                mountpoint=options['mount_point'], nothreads=True, foreground=True,
                allow_other=options['allow_other'],
                allow_root=options['allow_root']
            )
        except RuntimeError as exception:
            if options['allow_other'] or options['allow_root']:
                raise CommandError(
                    'Make sure \'user_allow_other\' is set in /etc/fuse.conf'
                )
            else:
                raise



from __future__ import absolute_import, unicode_literals

from datetime import timedelta

from celery.five import monotonic
from celery.task.control import inspect

from django.utils.encoding import force_text, python_2_unicode_compatible
from django.utils.timezone import now


@python_2_unicode_compatible
class TaskType(object):
    _registry = {}

    @classmethod
    def all(cls):
        return cls._registry.values()

    @classmethod
    def get(cls, name):
        return cls._registry[name]

    def __init__(self, name, label):
        self.name = name
        self.label = label
        self.__class__._registry[name] = self

    def __str__(self):
        return force_text(self.label)


@python_2_unicode_compatible
class Task(object):
    def __init__(self, task_type, kwargs):
        self.task_type = task_type
        self.kwargs = kwargs

    def __str__(self):
        return force_text(self.task_type)

    def get_time_started(self):
        time_start = self.kwargs.get('time_start')
        if time_start:
            return now() - timedelta(seconds=monotonic() - self.kwargs['time_start'])
        else:
            return None


@python_2_unicode_compatible
class CeleryQueue(object):
    _registry = {}
    _inspect_instance = inspect()

    @classmethod
    def all(cls):
        return sorted(
            cls._registry.values(), key=lambda instance: instance.label
        )

    @classmethod
    def get(cls, queue_name):
        return cls._registry[queue_name]

    def __init__(self, name, label, is_default_queue=False, transient=False):
        self.name = name
        self.label = label
        self.is_default_queue = is_default_queue
        self.is_transient = transient
        self.task_types = []
        self.__class__._registry[name] = self

    def __str__(self):
        return force_text(self.label)

    def add_task_type(self, *args, **kwargs):
        self.task_types.append(TaskType(*args, **kwargs))

    def get_active_tasks(self):
        return self._process_task_dictionary(
            task_dictionary=self.__class__._inspect_instance.active()
        )

    def get_reserved_tasks(self):
        return self._process_task_dictionary(
            task_dictionary=self.__class__._inspect_instance.reserved()
        )

    def get_scheduled_tasks(self):
        return self._process_task_dictionary(
            task_dictionary=self.__class__._inspect_instance.scheduled()
        )

    def _process_task_dictionary(self, task_dictionary):
        result = []
        for worker, tasks in task_dictionary.items():
            for task in tasks:
                if task['delivery_info']['routing_key'] == self.name:
                    task_type = TaskType.get(name=task['name'])
                    result.append(Task(task_type=task_type, kwargs=task))

        return result



from __future__ import unicode_literals
from django.utils.html import format_html_join


def get_metadata_string(document):
    """
    Return a formated representation of a document's metadata values
    """
    return format_html_join(
        '\n', '<div class="metadata-display"><b>{}: </b><span data-metadata-type="{}" data-pk="{}">{}</span></div>',
        ((document_metadata.metadata_type,document_metadata.metadata_type_id,document_metadata.id,document_metadata.value)
            for document_metadata in document.metadata.all()
        )
    )



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from common.queues import queue_tools
from task_manager.classes import CeleryQueue

queue_indexing = CeleryQueue(name='indexing', label=_('Indexing'))

queue_indexing.add_task_type(
    name='document_indexing.tasks.task_delete_empty',
    label=_('Delete empty index nodes')
)
queue_indexing.add_task_type(
    name='document_indexing.tasks.task_remove_document',
    label=_('Remove document')
)
queue_indexing.add_task_type(
    name='document_indexing.tasks.task_index_document',
    label=_('Index document')
)
queue_tools.add_task_type(
    name='document_indexing.tasks.task_rebuild_index',
    label=_('Rebuild index')
)



from __future__ import absolute_import, unicode_literals

import logging

from django.db import models, transaction
from django.template import Context, Template
from django.urls import reverse
from django.utils.encoding import force_text, python_2_unicode_compatible
from django.utils.translation import ugettext, ugettext_lazy as _

from mptt.fields import TreeForeignKey
from mptt.models import MPTTModel

from acls.models import AccessControlList
from documents.models import Document, DocumentType
from documents.permissions import permission_document_view
from lock_manager.runtime import locking_backend

from .managers import (
    DocumentIndexInstanceNodeManager, IndexManager, IndexInstanceNodeManager
)

logger = logging.getLogger(__name__)


@python_2_unicode_compatible
class Index(models.Model):
    """
    Parent model that defines an index and hold all the relationship for its
    template and instance when resolved.
    """
    label = models.CharField(
        max_length=128, unique=True, verbose_name=_('Label')
    )
    slug = models.SlugField(
        help_text=_(
            'This value will be used by other apps to reference this index.'
        ), max_length=128, unique=True, verbose_name=_('Slug')
    )
    enabled = models.BooleanField(
        default=True,
        help_text=_(
            'Causes this index to be visible and updated when document data '
            'changes.'
        ),
        verbose_name=_('Enabled')
    )
    document_types = models.ManyToManyField(
        DocumentType, verbose_name=_('Document types')
    )

    objects = IndexManager()

    class Meta:
        ordering = ('label',)
        verbose_name = _('Index')
        verbose_name_plural = _('Indexes')

    def __str__(self):
        return self.label

    def get_absolute_url(self):
        try:
            return reverse(
                'indexing:index_instance_node_view',
                args=(self.instance_root.pk,)
            )
        except IndexInstanceNode.DoesNotExist:
            return '#'

    def save(self, *args, **kwargs):
        """
        Automatically create the root index template node
        """
        super(Index, self).save(*args, **kwargs)
        IndexTemplateNode.objects.get_or_create(parent=None, index=self)

    @property
    def instance_root(self):
        return self.template_root.index_instance_nodes.get()

    @property
    def template_root(self):
        return self.node_templates.get(parent=None)

    def get_document_types_names(self):
        return ', '.join(
            [
                force_text(document_type) for document_type in self.document_types.all()
            ] or ['None']
        )

    def index_document(self, document):
        logger.debug('Index; Indexing document: %s', document)
        self.template_root.index_document(document=document)

    def rebuild(self):
        """
        Delete and reconstruct the index by deleting of all its instance nodes
        and recreating them for the documents whose types are associated with
        this index
        """
        # Delete all index instance nodes by deleting the root index
        # instance node. All child index instance nodes will be cascade
        # deleted.
        try:
            self.instance_root.delete()
        except IndexInstanceNode.DoesNotExist:
            # Empty index, ignore this exception
            pass

        # Create the new root index instance node
        self.template_root.index_instance_nodes.create()

        # Re-index each document with a type associated with this index
        for document in Document.objects.filter(document_type__in=self.document_types.all()):
            # Evaluate each index template node for each document
            # associated with this index.
            self.index_document(document=document)


class IndexInstance(Index):
    def get_instance_node_count(self):
        try:
            return self.instance_root.get_descendant_count()
        except IndexInstanceNode.DoesNotExist:
            return 0

    def get_item_count(self, user):
        try:
            return self.instance_root.get_item_count(user=user)
        except IndexInstanceNode.DoesNotExist:
            return 0

    class Meta:
        proxy = True
        verbose_name = _('Index instance')
        verbose_name_plural = _('Index instances')


@python_2_unicode_compatible
class IndexTemplateNode(MPTTModel):
    """
    The template to generate an index. Each entry represents a level in a
    hierarchy of levels. Each level can contain further levels or a list of
    documents but not both.
    """
    parent = TreeForeignKey(
        'self', blank=True, null=True, on_delete=models.CASCADE
    )
    index = models.ForeignKey(
        Index, on_delete=models.CASCADE, related_name='node_templates',
        verbose_name=_('Index')
    )
    expression = models.TextField(
        help_text=_(
            'Enter a template to render. '
            'Use Django\'s default templating language '
            '(https://docs.djangoproject.com/en/1.7/ref/templates/builtins/)'
        ),
        verbose_name=_('Indexing expression')
    )
    enabled = models.BooleanField(
        default=True,
        help_text=_(
            'Causes this node to be visible and updated when document data '
            'changes.'
        ),
        verbose_name=_('Enabled')
    )
    link_documents = models.BooleanField(
        default=False,
        help_text=_(
            'Check this option to have this node act as a container for '
            'documents and not as a parent for further nodes.'
        ),
        verbose_name=_('Link documents')
    )

    class Meta:
        verbose_name = _('Index node template')
        verbose_name_plural = _('Indexes node template')

    def __str__(self):
        if self.is_root_node():
            return ugettext('Root')
        else:
            return self.expression

    def index_document(self, document, acquire_lock=True, index_instance_node_parent=None):
        # Avoid another process indexing this same document for the same
        # template node. This prevents this template node's index instance
        # nodes from being deleted while the template is evaluated and
        # documents added to it.
        if acquire_lock:
            lock = locking_backend.acquire_lock(
                'indexing:indexing_template_node_{}'.format(self.pk)
            )

        # Start transaction after the lock in case the locking backend uses
        # the database.
        try:
            with transaction.atomic():
                logger.debug(
                    'IndexTemplateNode; Indexing document: %s', document
                )

                logger.debug(
                    'Removing document "%s" from all index instance nodes',
                    document
                )
                for index_template_node in self.index_instance_nodes.all():
                    index_template_node.remove_document(
                        document=document, acquire_lock=False
                    )

                if not self.parent:
                    logger.debug(
                        'IndexTemplateNode; parent: creating empty root index '
                        'instance node'
                    )
                    index_instance_node, created = self.index_instance_nodes.get_or_create()

                    for child in self.get_children():
                        child.index_document(
                            document=document, acquire_lock=False,
                            index_instance_node_parent=index_instance_node
                        )
                elif self.enabled:
                    logger.debug('IndexTemplateNode; non parent: evaluating')
                    logger.debug('My parent template is: %s', self.parent)
                    logger.debug(
                        'My parent instance node is: %s',
                        index_instance_node_parent
                    )
                    logger.debug(
                        'IndexTemplateNode; Evaluating template: %s', self.expression
                    )

                    try:
                        context = Context({'document': document})
                        template = Template(self.expression)
                        result = template.render(context=context)
                    except Exception as exception:
                        logger.debug('Evaluating error: %s', exception)
                        error_message = _(
                            'Error indexing document: %(document)s; expression: '
                            '%(expression)s; %(exception)s'
                        ) % {
                            'document': document,
                            'expression': self.expression,
                            'exception': exception
                        }
                        logger.debug(error_message)
                    else:
                        logger.debug('Evaluation result: %s', result)

                        if result:
                            index_instance_node, created = self.index_instance_nodes.get_or_create(
                                parent=index_instance_node_parent,
                                value=result
                            )
                            if self.link_documents:
                                index_instance_node.documents.add(document)

                            for child in self.get_children():
                                child.index_document(
                                    document=document, acquire_lock=False,
                                    index_instance_node_parent=index_instance_node
                                )
        finally:
            if acquire_lock:
                lock.release()


@python_2_unicode_compatible
class IndexInstanceNode(MPTTModel):
    parent = TreeForeignKey(
        'self', blank=True, null=True, on_delete=models.CASCADE
    )
    index_template_node = models.ForeignKey(
        IndexTemplateNode, on_delete=models.CASCADE,
        related_name='index_instance_nodes',
        verbose_name=_('Index template node')
    )
    value = models.CharField(
        blank=True, db_index=True, max_length=128, verbose_name=_('Value')
    )
    documents = models.ManyToManyField(
        Document, related_name='index_instance_nodes',
        verbose_name=_('Documents')
    )

    objects = IndexInstanceNodeManager()

    class Meta:
        verbose_name = _('Index node instance')
        verbose_name_plural = _('Indexes node instances')

    def __str__(self):
        return self.value

    def get_absolute_url(self):
        return reverse('indexing:index_instance_node_view', args=(self.pk,))

    @property
    def children(self):
        # Convenience method for serializer
        return self.get_children()

    def index(self):
        return IndexInstance.objects.get(pk=self.index_template_node.index.pk)

    def get_item_count(self, user):
        if self.index_template_node.link_documents:
            queryset = AccessControlList.objects.filter_by_access(
                permission_document_view, user, queryset=self.documents
            )

            return queryset.count()
        else:
            return self.get_children().count()

    def get_full_path(self):
        result = []
        for node in self.get_ancestors(include_self=True):
            if node.is_root_node():
                result.append(force_text(self.index()))
            else:
                result.append(force_text(node))

        return ' / '.join(result)

    def delete_empty(self, acquire_lock=True):
        """
        The argument `acquire_lock` controls whether or not this method
        acquires or lock. The case for this is to acquire when called directly
        or not to acquire when called as part of a larger index process
        that already has a lock
        """
        # Prevent another process to work on this node. We use the node's
        # parent template node for the lock
        if acquire_lock:
            lock = locking_backend.acquire_lock(
                'indexing:indexing_template_node_{}'.format(
                    self.index_template_node.pk
                )
            )
        # Start transaction after the lock in case the locking backend uses
        # the database.
        with transaction.atomic():
            if self.documents.count() == 0 and self.get_children().count() == 0:
                if self.parent:
                    self.delete()
                    self.parent.delete_empty(acquire_lock=False)
            if acquire_lock:
                lock.release()

    def remove_document(self, document, acquire_lock=True):
        """
        The argument `acquire_lock` controls whether or not this method
        acquires or lock. The case for this is to acquire when called directly
        or not to acquire when called as part of a larger index process
        that already has a lock
        """
        # Prevent another process to work on this node. We use the node's
        # parent template node for the lock
        if acquire_lock:
            lock = locking_backend.acquire_lock(
                'indexing:indexing_template_node_{}'.format(
                    self.index_template_node.pk
                )
            )
        self.documents.remove(document)
        self.delete_empty(acquire_lock=False)

        if acquire_lock:
            lock.release()


class DocumentIndexInstanceNode(IndexInstanceNode):
    objects = DocumentIndexInstanceNodeManager()

    class Meta:
        proxy = True
        verbose_name = _('Document index node instance')
        verbose_name_plural = _('Document indexes node instances')



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from permissions import PermissionNamespace

namespace = PermissionNamespace('task_manager', _('Task manager'))

permission_task_view = namespace.add_permission(
    name='task_view', label=_('View tasks')
)



from __future__ import absolute_import, unicode_literals

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core import mail
from django.test import override_settings
from django.urls import reverse

from common.tests import BaseTestCase
from smart_settings.classes import Namespace
from user_management.tests.literals import (
    TEST_ADMIN_EMAIL, TEST_ADMIN_PASSWORD, TEST_USER_PASSWORD_EDITED,
    TEST_ADMIN_USERNAME
)

from ..settings import setting_maximum_session_length

from .literals import TEST_EMAIL_AUTHENTICATION_BACKEND


class UserLoginTestCase(BaseTestCase):
    """
    Test that users can login via the supported authentication methods
    """

    def setUp(self):
        super(UserLoginTestCase, self).setUp()

        self.admin_user = get_user_model().objects.create_superuser(
            username=TEST_ADMIN_USERNAME, email=TEST_ADMIN_EMAIL,
            password=TEST_ADMIN_PASSWORD
        )
        Namespace.invalidate_cache_all()

    @override_settings(AUTHENTICATION_LOGIN_METHOD='username')
    def test_normal_behavior(self):
        response = self.client.get(reverse('documents:document_list'))
        self.assertRedirects(
            response,
            'http://testserver/authentication/login/?next=/documents/list/'
        )

    @override_settings(AUTHENTICATION_LOGIN_METHOD='username')
    def test_username_login(self):
        logged_in = self.client.login(
            username=TEST_ADMIN_USERNAME, password=TEST_ADMIN_PASSWORD
        )
        self.assertTrue(logged_in)
        response = self.client.get(reverse('documents:document_list'))
        # We didn't get redirected to the login URL
        self.assertEqual(response.status_code, 200)

    @override_settings(AUTHENTICATION_LOGIN_METHOD='email')
    def test_email_login(self):
        with self.settings(AUTHENTICATION_BACKENDS=(TEST_EMAIL_AUTHENTICATION_BACKEND,)):
            logged_in = self.client.login(
                username=TEST_ADMIN_USERNAME, password=TEST_ADMIN_PASSWORD
            )
            self.assertFalse(logged_in)

            logged_in = self.client.login(
                email=TEST_ADMIN_EMAIL, password=TEST_ADMIN_PASSWORD
            )
            self.assertTrue(logged_in)

            response = self.client.get(reverse('documents:document_list'))
            # We didn't get redirected to the login URL
            self.assertEqual(response.status_code, 200)

    @override_settings(AUTHENTICATION_LOGIN_METHOD='username')
    def test_username_login_via_views(self):
        response = self.client.get(reverse('documents:document_list'))
        self.assertRedirects(
            response,
            'http://testserver/authentication/login/?next=/documents/list/'
        )

        response = self.client.post(
            reverse(settings.LOGIN_URL), {
                'username': TEST_ADMIN_USERNAME,
                'password': TEST_ADMIN_PASSWORD
            }
        )
        response = self.client.get(reverse('documents:document_list'))
        # We didn't get redirected to the login URL
        self.assertEqual(response.status_code, 200)

    @override_settings(AUTHENTICATION_LOGIN_METHOD='email')
    def test_email_login_via_views(self):
        with self.settings(AUTHENTICATION_BACKENDS=(TEST_EMAIL_AUTHENTICATION_BACKEND,)):
            response = self.client.get(reverse('documents:document_list'))
            self.assertRedirects(
                response,
                'http://testserver/authentication/login/?next=/documents/list/'
            )

            response = self.client.post(
                reverse(settings.LOGIN_URL), {
                    'email': TEST_ADMIN_EMAIL, 'password': TEST_ADMIN_PASSWORD
                }, follow=True
            )
            self.assertEqual(response.status_code, 200)

            response = self.client.get(reverse('documents:document_list'))
            # We didn't get redirected to the login URL
            self.assertEqual(response.status_code, 200)

    @override_settings(AUTHENTICATION_LOGIN_METHOD='username')
    def test_username_remember_me(self):
        response = self.client.post(
            reverse(settings.LOGIN_URL), {
                'username': TEST_ADMIN_USERNAME,
                'password': TEST_ADMIN_PASSWORD,
                'remember_me': True
            }, follow=True
        )

        response = self.client.get(reverse('documents:document_list'))
        self.assertEqual(response.status_code, 200)

        self.assertEqual(
            self.client.session.get_expiry_age(),
            setting_maximum_session_length.value
        )
        self.assertFalse(self.client.session.get_expire_at_browser_close())

    @override_settings(AUTHENTICATION_LOGIN_METHOD='username')
    def test_username_dont_remember_me(self):
        response = self.client.post(
            reverse(settings.LOGIN_URL), {
                'username': TEST_ADMIN_USERNAME,
                'password': TEST_ADMIN_PASSWORD,
                'remember_me': False
            }, follow=True
        )

        response = self.client.get(reverse('documents:document_list'))
        self.assertEqual(response.status_code, 200)

        self.assertTrue(self.client.session.get_expire_at_browser_close())

    @override_settings(AUTHENTICATION_LOGIN_METHOD='email')
    def test_email_remember_me(self):
        with self.settings(AUTHENTICATION_BACKENDS=(TEST_EMAIL_AUTHENTICATION_BACKEND,)):
            response = self.client.post(
                reverse(settings.LOGIN_URL), {
                    'email': TEST_ADMIN_EMAIL,
                    'password': TEST_ADMIN_PASSWORD,
                    'remember_me': True
                }, follow=True
            )

            response = self.client.get(reverse('documents:document_list'))
            self.assertEqual(response.status_code, 200)

            self.assertEqual(
                self.client.session.get_expiry_age(),
                setting_maximum_session_length.value
            )
            self.assertFalse(self.client.session.get_expire_at_browser_close())

    @override_settings(AUTHENTICATION_LOGIN_METHOD='email')
    def test_email_dont_remember_me(self):
        with self.settings(AUTHENTICATION_BACKENDS=(TEST_EMAIL_AUTHENTICATION_BACKEND,)):
            response = self.client.post(
                reverse(settings.LOGIN_URL), {
                    'email': TEST_ADMIN_EMAIL,
                    'password': TEST_ADMIN_PASSWORD,
                    'remember_me': False
                }, follow=True
            )

            response = self.client.get(reverse('documents:document_list'))
            self.assertEqual(response.status_code, 200)

            self.assertTrue(self.client.session.get_expire_at_browser_close())

    @override_settings(AUTHENTICATION_LOGIN_METHOD='username')
    def test_password_reset(self):
        response = self.client.post(
            reverse('authentication:password_reset_view'), {
                'email': TEST_ADMIN_EMAIL,
            }, follow=True
        )

        self.assertContains(
            response, text='Password reset email sent!', status_code=200
        )
        self.assertEqual(len(mail.outbox), 1)

        uid_token = mail.outbox[0].body.replace('\n', '').split('/')

        response = self.client.post(
            reverse('authentication:password_reset_confirm_view', args=uid_token[-3:-1]), {
                'new_password1': TEST_USER_PASSWORD_EDITED,
                'new_password2': TEST_USER_PASSWORD_EDITED,
            }, follow=True
        )

        self.assertContains(
            response, text='Password reset complete!', status_code=200
        )

        response = self.client.post(
            reverse(settings.LOGIN_URL), {
                'username': TEST_ADMIN_USERNAME,
                'password': TEST_USER_PASSWORD_EDITED,
                'remember_me': True
            }, follow=True
        )

        response = self.client.get(reverse('documents:document_list'))
        self.assertEqual(response.status_code, 200)



from __future__ import absolute_import, unicode_literals

import os

from common.settings import setting_paginate_by
from common.tests import BaseTestCase

from .literals import ENVIRONMENT_TEST_NAME, ENVIRONMENT_TEST_VALUE


class ClassesTestCase(BaseTestCase):
    def test_environment_variable(self):
        os.environ[
            'MAYAN_{}'.format(ENVIRONMENT_TEST_NAME)
        ] = ENVIRONMENT_TEST_VALUE
        self.assertTrue(setting_paginate_by.value, ENVIRONMENT_TEST_VALUE)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-14 21:33
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('mailer', '0003_auto_20170703_1535'),
    ]

    operations = [
        migrations.AlterField(
            model_name='usermailer',
            name='default',
            field=models.BooleanField(default=True, help_text='If default, this mailing profile will be pre-selected on the document mailing form.', verbose_name='Default'),
        ),
    ]



from __future__ import unicode_literals

from common.classes import PropertyHelper


class DocumentStateHelper(PropertyHelper):
    @staticmethod
    @property
    def constructor(*args, **kwargs):
        return DocumentStateHelper(*args, **kwargs)

    def get_result(self, name):
        return self.instance.workflows.get(workflow__internal_name=name)



from __future__ import unicode_literals


class BaseCommonException(Exception):
    """
    Base exception for the common app
    """
    pass


class NotLatestVersion(BaseCommonException):
    """
    The installed version is not the latest available version
    """
    def __init__(self, upstream_version):
        self.upstream_version = upstream_version



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from task_manager.classes import CeleryQueue

queue_sources = CeleryQueue(
    name='sources', label=_('Sources'), transient=True
)
queue_sources_periodic = CeleryQueue(
    name='sources_periodic', label=_('Sources periodic')
)

queue_sources_periodic.add_task_type(
    name='sources.tasks.task_check_interval_source',
    label=_('Check interval source')
)

queue_sources.add_task_type(
    name='sources.tasks.task_source_handle_upload',
    label=_('Handle upload')
)
queue_sources.add_task_type(
    name='sources.tasks.task_upload_document',
    label=_('Upload document')
)



from __future__ import unicode_literals

DEFAULT_ZOOM_LEVEL = 100
DEFAULT_ROTATION = 0
DEFAULT_PAGE_NUMBER = 1
DEFAULT_FILE_FORMAT = 'JPEG'
DEFAULT_LIBREOFFICE_PATH = '/usr/bin/libreoffice'

DIMENSION_SEPARATOR = 'x'



from __future__ import unicode_literals

from kombu import Exchange, Queue

from django.apps import apps
from django.utils.translation import ugettext_lazy as _

from acls import ModelPermission
from common import (
    MayanAppConfig, menu_object, menu_multi_item, menu_secondary, menu_setup,
    menu_tools
)
from common.widgets import two_state_template
from mayan.celery import app
from navigation import SourceColumn

from .classes import MailerBackend
from .links import (
    link_send_document_link, link_send_document, link_send_multiple_document,
    link_send_multiple_document_link, link_system_mailer_error_log,
    link_user_mailer_create, link_user_mailer_delete, link_user_mailer_edit,
    link_user_mailer_list, link_user_mailer_log_list, link_user_mailer_setup,
    link_user_mailer_test
)
from .permissions import (
    permission_mailing_link, permission_mailing_send_document,
    permission_user_mailer_delete, permission_user_mailer_edit,
    permission_user_mailer_use, permission_user_mailer_view,
)
from .queues import *  # NOQA


class MailerApp(MayanAppConfig):
    has_tests = True
    name = 'mailer'
    verbose_name = _('Mailer')

    def ready(self):
        super(MailerApp, self).ready()

        Document = apps.get_model(
            app_label='documents', model_name='Document'
        )

        LogEntry = self.get_model('LogEntry')
        UserMailer = self.get_model('UserMailer')

        MailerBackend.initialize()

        SourceColumn(
            source=LogEntry, label=_('Date and time'), attribute='datetime'
        )
        SourceColumn(
            source=LogEntry, label=_('Message'), attribute='message'
        )
        SourceColumn(
            source=UserMailer, label=_('Label'), attribute='label'
        )
        SourceColumn(
            source=UserMailer, label=_('Default?'),
            func=lambda context: two_state_template(
                context['object'].default
            )
        )
        SourceColumn(
            source=UserMailer, label=_('Enabled?'),
            func=lambda context: two_state_template(
                context['object'].enabled
            )
        )
        SourceColumn(
            source=UserMailer, label=_('Label'), attribute='backend_label'
        )

        ModelPermission.register(
            model=Document, permissions=(
                permission_mailing_link, permission_mailing_send_document
            )
        )

        ModelPermission.register(
            model=UserMailer, permissions=(
                permission_user_mailer_delete, permission_user_mailer_edit,
                permission_user_mailer_view, permission_user_mailer_use
            )
        )

        app.conf.CELERY_QUEUES.append(
            Queue('mailing', Exchange('mailing'), routing_key='mailing'),
        )

        app.conf.CELERY_ROUTES.update(
            {
                'mailer.tasks.task_send_document': {
                    'queue': 'mailing'
                },
            }
        )

        menu_multi_item.bind_links(
            links=(
                link_send_multiple_document, link_send_multiple_document_link
            ), sources=(Document,)
        )

        menu_object.bind_links(
            links=(
                link_send_document_link, link_send_document
            ), sources=(Document,)
        )

        menu_object.bind_links(
            links=(
                link_user_mailer_edit, link_user_mailer_log_list,
                link_user_mailer_test, link_user_mailer_delete,
            ), sources=(UserMailer,)
        )

        menu_secondary.bind_links(
            links=(
                link_user_mailer_list, link_user_mailer_create,
            ), sources=(
                UserMailer, 'mailer:user_mailer_list',
                'mailer:user_mailer_create'
            )
        )

        menu_tools.bind_links(links=(link_system_mailer_error_log,))

        menu_setup.bind_links(links=(link_user_mailer_setup,))



from __future__ import absolute_import, unicode_literals

from datetime import timedelta
import logging
import os

from kombu import Exchange, Queue

from django import apps
from django.conf import settings
from django.conf.urls import include, url
from django.contrib.auth.signals import user_logged_in
from django.db.models.signals import post_save
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _

from mayan.celery import app
from navigation.classes import Separator, Text
from rest_api.classes import APIEndPoint

from .handlers import (
    user_locale_profile_session_config, user_locale_profile_create
)
from .links import (
    link_about, link_check_version, link_code, link_current_user_details,
    link_current_user_edit, link_current_user_locale_profile_edit,
    link_documentation, link_filters, link_forum, link_license,
    link_packages_licenses, link_setup, link_support, link_tools
)
from .literals import DELETE_STALE_UPLOADS_INTERVAL
from .menus import menu_about, menu_main, menu_tools, menu_user
from .licenses import *  # NOQA
from .queues import *  # NOQA - Force queues registration
from .settings import setting_auto_logging, setting_production_error_log_path
from .tasks import task_delete_stale_uploads  # NOQA - Force task registration

logger = logging.getLogger(__name__)


class MayanAppConfig(apps.AppConfig):
    app_url = None
    app_namespace = None

    def ready(self):
        from mayan.urls import urlpatterns

        if self.app_url:
            top_url = '{}/'.format(self.app_url)
        elif self.app_url is not None:
            top_url = ''
        else:
            top_url = '{}/'.format(self.name)

        try:
            urlpatterns += url(
                r'^{}'.format(top_url),
                include(
                    '{}.urls'.format(self.name),
                    namespace=self.app_namespace or self.name
                )
            ),
        except ImportError as exception:
            if force_text(exception) != 'No module named urls':
                logger.error(
                    'Import time error when running AppConfig.ready(). Check '
                    'apps.py, urls.py, views.py, etc.'
                )
                raise exception


class CommonApp(MayanAppConfig):
    app_url = ''
    has_tests = True
    name = 'common'
    verbose_name = _('Common')

    @staticmethod
    def get_user_label_text(context):
        if not context['request'].user.is_authenticated:
            return _('Anonymous')
        else:
            return context['request'].user.get_full_name() or context['request'].user

    def ready(self):
        super(CommonApp, self).ready()

        APIEndPoint(app=self, version_string='1')

        app.conf.CELERYBEAT_SCHEDULE.update(
            {
                'task_delete_stale_uploads': {
                    'task': 'common.tasks.task_delete_stale_uploads',
                    'schedule': timedelta(
                        seconds=DELETE_STALE_UPLOADS_INTERVAL
                    ),
                },
            }
        )

        app.conf.CELERY_QUEUES.extend(
            (
                Queue('default', Exchange('default'), routing_key='default'),
                Queue('tools', Exchange('tools'), routing_key='tools'),
                Queue(
                    'common_periodic', Exchange('common_periodic'),
                    routing_key='common_periodic', delivery_mode=1
                ),
            )
        )

        app.conf.CELERY_DEFAULT_QUEUE = 'default'

        app.conf.CELERY_ROUTES.update(
            {
                'common.tasks.task_delete_stale_uploads': {
                    'queue': 'common_periodic'
                },
            }
        )
        menu_user.bind_links(
            links=(
                Text(text=CommonApp.get_user_label_text), Separator(),
                link_current_user_details, link_current_user_edit,
                link_current_user_locale_profile_edit,
                Separator()
            )
        )

        menu_about.bind_links(
            links=(
                link_tools, link_setup, link_about, link_support,
                link_documentation, link_forum, link_code, link_license,
                link_packages_licenses, link_check_version
            )
        )

        menu_main.bind_links(links=(menu_about, menu_user,), position=99)

        menu_tools.bind_links(
            links=(link_filters,)
        )

        post_save.connect(
            user_locale_profile_create,
            dispatch_uid='user_locale_profile_create',
            sender=settings.AUTH_USER_MODEL
        )
        user_logged_in.connect(
            user_locale_profile_session_config,
            dispatch_uid='user_locale_profile_session_config'
        )
        self.setup_auto_logging()

    def setup_auto_logging(self):
        if setting_auto_logging.value:
            if settings.DEBUG:
                level = 'DEBUG'
                handlers = ['console']
            else:
                level = 'ERROR'
                handlers = ['console', 'logfile']

            loggers = {}
            for project_app in apps.apps.get_app_configs():
                loggers[project_app.name] = {
                    'handlers': handlers,
                    'propagate': True,
                    'level': level,
                }

            logging.config.dictConfig(
                {
                    'version': 1,
                    'disable_existing_loggers': True,
                    'formatters': {
                        'intermediate': {
                            'format': '%(name)s <%(process)d> [%(levelname)s] "%(funcName)s() %(message)s"'
                        },
                    },
                    'handlers': {
                        'console': {
                            'level': 'DEBUG',
                            'class': 'logging.StreamHandler',
                            'formatter': 'intermediate'
                        },
                        'logfile': {
                            'class': 'logging.handlers.WatchedFileHandler',
                            'filename': setting_production_error_log_path.value,
                        },
                    },
                    'loggers': loggers
                }
            )



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from task_manager.classes import CeleryQueue

queue_statistics = CeleryQueue(
    name='statistics', label=_('Statistics'), transient=True
)

queue_statistics.add_task_type(
    name='statistics.tasks.task_execute_statistic',
    label=_('Execute statistic')
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import converter.validators


class Migration(migrations.Migration):

    dependencies = [
        ('converter', '0003_auto_20150704_0731'),
    ]

    operations = [
        migrations.AlterField(
            model_name='transformation',
            name='arguments',
            field=models.TextField(
                help_text='Enter the arguments for the transformation as a '
                'YAML dictionary. ie: {"degrees": 180}',
                blank=True, verbose_name='Arguments',
                validators=[converter.validators.YAMLValidator()]
            ),
            preserve_default=True,
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-06-30 20:40
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('sources', '0015_auto_20170206_0835'),
    ]

    operations = [
        migrations.AlterField(
            model_name='sanescanner',
            name='resolution',
            field=models.PositiveIntegerField(blank=True, help_text='Sets the resolution of the scanned image in DPI (dots per inch). Typical value is 200. If this option is not supported by your scanner, leave it blank.', null=True, verbose_name='Resolution'),
        ),
    ]



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _

from task_manager.classes import CeleryQueue

queue_default = CeleryQueue(
    name='default', label=_('Default'), is_default_queue=True
)
queue_tools = CeleryQueue(name='tools', label=_('Tools'))
queue_common_periodic = CeleryQueue(
    name='common_periodic', label=_('Common periodic'), transient=True
)
queue_common_periodic.add_task_type(
    name='common.tasks.task_delete_stale_uploads',
    label=_('Delete stale uploads')
)



from __future__ import unicode_literals

from importlib import import_module
import logging

from django.apps import apps
from django.utils import six
from django.utils.encoding import force_text

logger = logging.getLogger(__name__)


__ALL__ = ('MailerBackend',)


class MailerBackendMetaclass(type):
    _registry = {}

    def __new__(mcs, name, bases, attrs):
        new_class = super(MailerBackendMetaclass, mcs).__new__(
            mcs, name, bases, attrs
        )
        if not new_class.__module__ == 'mailer.classes':
            mcs._registry[
                '{}.{}'.format(new_class.__module__, name)
            ] = new_class

        return new_class


class MailerBackendBase(object):
    """
    Base class for the mailing backends. This class is mainly a wrapper
    for other Django backends that adds a few metadata to specify the
    fields it needs to be instanciated at runtime.

    The fields attribute is a list of dictionaries with the format:
    {
        'name': ''  # Field internal name
        'label': ''  # Label to show to users
        'class': ''  # Field class to use. Field classes are Python dot
                       paths to Django's form fields.
        'initial': ''  # Field initial value
        'default': ''  # Default value.
    }

    """
    class_path = ''  # Dot path to the actual class that will handle the mail
    fields = ()


class MailerBackend(six.with_metaclass(MailerBackendMetaclass, MailerBackendBase)):
    @classmethod
    def get(cls, name):
        return cls._registry[name]

    @classmethod
    def get_all(cls):
        return cls._registry

    @staticmethod
    def initialize():
        for app in apps.get_app_configs():
            try:
                import_module('{}.mailers'.format(app.name))
            except ImportError as exception:
                if force_text(exception) != 'No module named mailers':
                    logger.error(
                        'Error importing %s mailers.py file; %s', app.name,
                        exception
                    )



from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from navigation import Link

from .permissions import permission_task_view

link_task_manager = Link(
    icon='fa fa-braille', permissions=(permission_task_view,),
    text=_('Task manager'), view='task_manager:queue_list'
)
link_queue_list = Link(
    icon='fa fa-braille', permissions=(permission_task_view,),
    text=_('Background task queues'), view='task_manager:queue_list'
)
link_queue_active_task_list = Link(
    args='resolved_object.name', permissions=(permission_task_view,),
    text=_('Active tasks'), view='task_manager:queue_active_task_list'
)
link_queue_reserved_task_list = Link(
    args='resolved_object.name', permissions=(permission_task_view,),
    text=_('Reserved tasks'), view='task_manager:queue_reserved_task_list'
)
link_queue_scheduled_task_list = Link(
    args='resolved_object.name', permissions=(permission_task_view,),
    text=_('Scheduled tasks'), view='task_manager:queue_scheduled_task_list'
)
